﻿using System;
using System.Windows.Forms;
using ThinkGeo.MapSuite;
using ThinkGeo.MapSuite.Layers;
using ThinkGeo.MapSuite.Shapes;
using ThinkGeo.MapSuite.Styles;
using ThinkGeo.MapSuite.WinForms;

namespace _8873
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        Timer timer = new Timer();
        bool stopRefresh = false;
        int zoomDelta = 0;
        double currentScale = 0;

        private void Form1_Load(object sender, EventArgs e)
        {
            timer.Tick += Timer_Tick;
            timer.Interval = 1000; // After stop wheel, it will fire refresh

            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;

            LayerOverlay layerOverlay = new LayerOverlay();
            ShapeFileFeatureLayer shapeFileLayer = new ShapeFileFeatureLayer(@"..\..\AppData\states.shp");
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = WorldStreetsAreaStyles.Military();
            shapeFileLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;
            layerOverlay.Layers.Add(shapeFileLayer);
            winformsMap1.Overlays.Add(layerOverlay);

            shapeFileLayer.Open();
            winformsMap1.CurrentExtent = shapeFileLayer.GetBoundingBox();

            winformsMap1.MouseWheel += WinformsMap1_MouseWheel;
            winformsMap1.CurrentExtentChanging += WinformsMap1_CurrentExtentChanging;
            winformsMap1.Refresh();
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            timer.Stop();
            stopRefresh = false;

            double targetScale = GetTargetZoomLevelScale(currentScale, zoomDelta / 120);
            winformsMap1.ZoomToScale(targetScale);

            zoomDelta = 0;

            winformsMap1.Refresh();
        }

        private double GetTargetZoomLevelScale(double currentScale, int delta)
        {
            System.Collections.ObjectModel.Collection<double> scales = new System.Collections.ObjectModel.Collection<double>();

            foreach (ZoomLevel level in winformsMap1.ZoomLevelSet.GetZoomLevels())
            {
                scales.Add(level.Scale);
            }

            int currentIndex = ExtentHelper.GetSnappedZoomLevelIndex(currentScale, scales);
            int targetIndex = currentIndex + delta;

            if (targetIndex < 1) targetIndex = 1;
            if (targetIndex > 20) targetIndex = 20;

            return scales[targetIndex - 1];
        }

        private void WinformsMap1_CurrentExtentChanging(object sender, CurrentExtentChangingWinformsMapEventArgs e)
        {
            currentScale = winformsMap1.CurrentScale;

            if (stopRefresh)
            {
                e.Cancel = true;
            }
        }

        private void WinformsMap1_MouseWheel(object sender, MouseEventArgs e)
        {
            timer.Stop();
            stopRefresh = true;
            zoomDelta += e.Delta;
            timer.Start();
        }
    }
}
